From: Jacopo Mondi <jacopo@jmondi.org>
To: "Krzysztof Hałasa" <khalasa@piap.pl>,
"Mauro Carvalho Chehab" <mchehab@kernel.org>,
"Sakari Ailus" <sakari.ailus@iki.fi>,
"Laurent Pinchart" <laurent.pinchart@ideasonboard.com>
Cc: Jacopo Mondi <jacopo@jmondi.org>, linux-media@vger.kernel.org
Subject: [PATCH v2 07/10] media: ar0521: Adjust exposure and blankings limits
Date: Sat, 22 Oct 2022 11:20:12 +0200 [thread overview]
Message-ID: <20221022092015.208592-8-jacopo@jmondi.org> (raw)
In-Reply-To: <20221022092015.208592-1-jacopo@jmondi.org>
Adjust the control limits for V4L2_CID_VBLANK, V4L2_CID_HBLANK and
V4L2_CID_EXPOSURE when a new format is applied to the sensor.
Also update the exposure control when a new blanking value is applied.
Also change the controls initialization to use valid values for the
default format.
Signed-off-by: Jacopo Mondi <jacopo@jmondi.org>
---
drivers/media/i2c/ar0521.c | 79 ++++++++++++++++++++++++++++----------
1 file changed, 59 insertions(+), 20 deletions(-)
diff --git a/drivers/media/i2c/ar0521.c b/drivers/media/i2c/ar0521.c
index 2310346f11d5..854f4ccd9c3d 100644
--- a/drivers/media/i2c/ar0521.c
+++ b/drivers/media/i2c/ar0521.c
@@ -40,7 +40,8 @@
#define AR0521_WIDTH_BLANKING_MIN 572u
#define AR0521_HEIGHT_BLANKING_MIN 38u /* must be even */
-#define AR0521_TOTAL_WIDTH_MIN 2968u
+#define AR0521_TOTAL_HEIGHT_MAX 65535u /* max_frame_lenght_lines */
+#define AR0521_TOTAL_WIDTH_MAX 65532u /* max_line_lenght_pck */
#define AR0521_ANA_GAIN_MIN 0x00
#define AR0521_ANA_GAIN_MAX 0x3f
@@ -125,8 +126,6 @@ struct ar0521_dev {
struct v4l2_mbus_framefmt fmt;
struct ar0521_ctrls ctrls;
unsigned int lane_count;
- u16 total_width;
- u16 total_height;
struct {
u16 pre;
u16 mult;
@@ -483,7 +482,8 @@ static int ar0521_set_fmt(struct v4l2_subdev *sd,
struct v4l2_subdev_format *format)
{
struct ar0521_dev *sensor = to_ar0521_dev(sd);
- int ret = 0;
+ int exposure_max, exposure_val;
+ int max_vblank, max_hblank;
ar0521_adj_fmt(&format->format);
@@ -494,33 +494,64 @@ static int ar0521_set_fmt(struct v4l2_subdev *sd,
fmt = v4l2_subdev_get_try_format(sd, sd_state, 0 /* pad */);
*fmt = format->format;
- } else {
- sensor->fmt = format->format;
- ar0521_calc_mode(sensor);
+
+ mutex_unlock(&sensor->lock);
+
+ return 0;
}
+ sensor->fmt = format->format;
+ ar0521_calc_mode(sensor);
+
+ /*
+ * Update the exposure and blankings limits. Blankings are also reset
+ * to the minimum.
+ */
+ max_hblank = AR0521_TOTAL_WIDTH_MAX - sensor->fmt.width;
+ __v4l2_ctrl_modify_range(sensor->ctrls.hblank,
+ sensor->ctrls.hblank->minimum,
+ max_hblank, sensor->ctrls.hblank->step,
+ sensor->ctrls.hblank->minimum);
+ __v4l2_ctrl_s_ctrl(sensor->ctrls.vblank, sensor->ctrls.hblank->minimum);
+
+ max_vblank = AR0521_TOTAL_HEIGHT_MAX - sensor->fmt.height;
+ __v4l2_ctrl_modify_range(sensor->ctrls.vblank,
+ sensor->ctrls.vblank->minimum,
+ max_vblank, sensor->ctrls.vblank->step,
+ sensor->ctrls.vblank->minimum);
+ __v4l2_ctrl_s_ctrl(sensor->ctrls.vblank, sensor->ctrls.vblank->minimum);
+
+ exposure_max = sensor->fmt.height + AR0521_HEIGHT_BLANKING_MIN - 4;
+ exposure_val = min(sensor->ctrls.exposure->val, exposure_max);
+ __v4l2_ctrl_modify_range(sensor->ctrls.exposure,
+ sensor->ctrls.exposure->minimum,
+ exposure_max, sensor->ctrls.exposure->step,
+ exposure_val);
+ __v4l2_ctrl_s_ctrl(sensor->ctrls.exposure, exposure_val);
+
mutex_unlock(&sensor->lock);
- return ret;
+
+ return 0;
}
static int ar0521_s_ctrl(struct v4l2_ctrl *ctrl)
{
struct v4l2_subdev *sd = ctrl_to_sd(ctrl);
struct ar0521_dev *sensor = to_ar0521_dev(sd);
+ int exp_max;
+ int exp_val;
int ret;
/* v4l2_ctrl_lock() locks our own mutex */
switch (ctrl->id) {
- case V4L2_CID_HBLANK:
case V4L2_CID_VBLANK:
- sensor->total_width = sensor->fmt.width +
- sensor->ctrls.hblank->val;
- sensor->total_height = sensor->fmt.width +
- sensor->ctrls.vblank->val;
- break;
- default:
- ret = -EINVAL;
+ exp_max = sensor->fmt.height + ctrl->val - 4;
+ exp_val = min(sensor->ctrls.exposure->val, exp_max);
+ __v4l2_ctrl_modify_range(sensor->ctrls.exposure,
+ sensor->ctrls.exposure->minimum,
+ exp_max, sensor->ctrls.exposure->step,
+ exp_val);
break;
}
@@ -584,6 +615,7 @@ static int ar0521_init_controls(struct ar0521_dev *sensor)
const struct v4l2_ctrl_ops *ops = &ar0521_ctrl_ops;
struct ar0521_ctrls *ctrls = &sensor->ctrls;
struct v4l2_ctrl_handler *hdl = &ctrls->handler;
+ int max_vblank, max_hblank, exposure_max;
int ret;
v4l2_ctrl_handler_init(hdl, 32);
@@ -604,11 +636,17 @@ static int ar0521_init_controls(struct ar0521_dev *sensor)
-512, 511, 1, 0);
v4l2_ctrl_cluster(3, &ctrls->gain);
+ /* Initialize blanking limits using the default 2592x1944 format. */
+ max_hblank = AR0521_TOTAL_WIDTH_MAX - AR0521_WIDTH_MAX;
ctrls->hblank = v4l2_ctrl_new_std(hdl, ops, V4L2_CID_HBLANK,
- AR0521_WIDTH_BLANKING_MIN, 4094, 1,
+ AR0521_WIDTH_BLANKING_MIN,
+ max_hblank, 1,
AR0521_WIDTH_BLANKING_MIN);
+
+ max_vblank = AR0521_TOTAL_HEIGHT_MAX - AR0521_HEIGHT_MAX;
ctrls->vblank = v4l2_ctrl_new_std(hdl, ops, V4L2_CID_VBLANK,
- AR0521_HEIGHT_BLANKING_MIN, 4094, 2,
+ AR0521_HEIGHT_BLANKING_MIN,
+ max_vblank, 2,
AR0521_HEIGHT_BLANKING_MIN);
v4l2_ctrl_cluster(2, &ctrls->hblank);
@@ -618,9 +656,10 @@ static int ar0521_init_controls(struct ar0521_dev *sensor)
AR0521_PIXEL_CLOCK_MAX, 1,
AR0521_PIXEL_CLOCK_RATE);
- /* Manual exposure time */
+ /* Manual exposure time: max exposure time = visible + blank - 4 */
+ exposure_max = AR0521_HEIGHT_MAX + AR0521_HEIGHT_BLANKING_MIN - 4;
ctrls->exposure = v4l2_ctrl_new_std(hdl, ops, V4L2_CID_EXPOSURE, 0,
- 65535, 1, 360);
+ exposure_max, 1, 360);
v4l2_ctrl_new_int_menu(hdl, ops, V4L2_CID_LINK_FREQ,
ARRAY_SIZE(ar0521_link_frequencies) - 1,
--
2.37.3
next prev parent reply other threads:[~2022-10-22 10:20 UTC|newest]
Thread overview: 20+ messages / expand[flat|nested] mbox.gz Atom feed top
2022-10-22 9:20 [PATCH v2 00/10] media: ar0521: Add analog gain, rework clock tree Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 01/10] media: ar0521: Implement enum_frame_sizes Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 02/10] media: ar0521: Add V4L2_CID_ANALOG_GAIN Jacopo Mondi
2022-10-24 12:13 ` Dave Stevenson
2022-10-24 12:31 ` Jacopo Mondi
2022-10-25 14:02 ` Sakari Ailus
2022-10-22 9:20 ` [PATCH v2 03/10] media: ar0521: Set maximum resolution to 2592x1944 Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 04/10] media: ar0521: Rework PLL computation Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 05/10] media: ar0521: Refuse unsupported controls Jacopo Mondi
2022-10-24 13:00 ` Dave Stevenson
2022-10-24 13:10 ` Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 06/10] media: ar0521: Add LINK_FREQ control Jacopo Mondi
2022-10-24 15:44 ` Dave Stevenson
2022-10-22 9:20 ` Jacopo Mondi [this message]
2022-10-24 17:47 ` [PATCH v2 07/10] media: ar0521: Adjust exposure and blankings limits Dave Stevenson
2022-10-25 17:45 ` Dave Stevenson
2022-10-27 7:10 ` Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 08/10] media: ar0521: Setup controls at s_stream time Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 09/10] media: ar0521: Rework startup sequence Jacopo Mondi
2022-10-22 9:20 ` [PATCH v2 10/10] media: ar0521: Tab-align definitions Jacopo Mondi
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20221022092015.208592-8-jacopo@jmondi.org \
--to=jacopo@jmondi.org \
--cc=khalasa@piap.pl \
--cc=laurent.pinchart@ideasonboard.com \
--cc=linux-media@vger.kernel.org \
--cc=mchehab@kernel.org \
--cc=sakari.ailus@iki.fi \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox